#pragma once

struct strv {
    int count;
    const char *data;
};

#define STRV_STATIC(str)                         \
    {                                            \
        .count = sizeof(str) - 1, .data = (str), \
    }

/* Create the NULL/empty string view  */
#define STRV_NULL strv_from_slice(NULL, 0)

#define STRV_FMT     "%.*s"                    /* Format string for a string view */
#define STRV_ARG(sv) ((sv).count), ((sv).data) /* Unpack the string view */

/* Create a string view from a slice or a C string. */
struct strv strv_from_slice(const char *data, int count);
struct strv strv_from_str(const char *);
/* Copy a string view */
struct strv strv_copy(const struct strv sv);

/* Copy back a string view to a C string */
_Bool strv_as_str(const struct strv, char *data, int len);

/* Convert strv to an integet, in decimal notation */
int strv_to_int(const struct strv);

/* Get the length of the string view */
int strv_len(struct strv sv);

/* Different ways of trimming the strings */
struct strv strv_trim_l(struct strv);
struct strv strv_trim_r(struct strv);
struct strv strv_trim(struct strv);

struct strv strv_take_left_while(struct strv, _Bool (*)(char));
struct strv strv_chop_by_delim(struct strv *, char);
_Bool strv_try_chop_by_delim(struct strv *, char delim, struct strv *chunk);
struct strv strv_chop_left(struct strv *, int);
struct strv strv_chop_right(struct strv *, int);
struct strv strv_chop_left_while(struct strv *, _Bool (*)(char));

/* Find substrings or characters in a string view. Returns 1 if found, 0
 * otherwise. For the index_of function the index of the character is placed
 * inside `index` if the pointer is not NULL. */
_Bool strv_index_of(struct strv sv, char c, int *index);
_Bool strv_starts_with(struct strv sv, struct strv prefix);
_Bool strv_ends_with(struct strv sv, struct strv suffix);

/* Compare two string views */
_Bool strv_equal(struct strv a, struct strv b);
_Bool strv_equal_nocase(struct strv a, struct strv b);

/* Private implementation of some functions to not include some headers */

char strv_ctoupper(char s);

char strv_ctolower(char s);

_Bool strv_memcmp(const void *ptr2, const void *ptr1, int count);

_Bool strv_isspace(char x);

int strv_strlen(const char *str);

/* Implementation of the functions */

struct strv strv_from_slice(const char *data, int count);

struct strv strv_from_str(const char *cstr);

_Bool strv_as_str(const struct strv sv, char *data, int len);

struct strv strv_trim_l(struct strv sv);

struct strv strv_trim_r(struct strv sv);

struct strv strv_trim(struct strv sv);

struct strv strv_chop_left(struct strv *sv, int n);

struct strv strv_chop_right(struct strv *sv, int n);

_Bool strv_index_of(struct strv sv, char c, int *index);

_Bool strv_try_chop_by_delim(struct strv *sv, char delim, struct strv *chunk);

struct strv strv_chop_by_delim(struct strv *sv, char delim);

_Bool strv_starts_with(struct strv sv, struct strv expected_prefix);

_Bool strv_ends_with(struct strv sv, struct strv expected_suffix);

_Bool strv_equal(struct strv a, struct strv b);

_Bool strv_equal_nocase(struct strv a, struct strv b);

struct strv strv_chop_left_while(struct strv *sv, _Bool (*cb)(char));

struct strv strv_take_left_while(struct strv sv, _Bool (*cb)(char));

int strv_len(struct strv sv);

struct strv strv_copy(const struct strv sv);

int strv_to_int(const struct strv sv_init);
