#if !defined(LKT_QUEUE_H__)
#define LKT_QUEUE_H__

#if defined(__cplusplus)
extern "C" {
#endif

/* Everything in this file is implemented in the header, so that modules don't
   have to compile the lib with another .c file. */

#include <lektor/common.h>

enum {
    __LKT_PLAY_STOP   = 0,
    __LKT_PLAY_PLAY   = 1,
    __LKT_PLAY_PAUSE  = 2,
    __LKT_PLAY_TOGGLE = 3,
};

enum {
    __LKT_DB_UPDATING_FINISHED = 0, /* Not updating       */
    __LKT_DB_UPDATING_PROGRESS = 1, /* Update in progress */
};

#if defined(__cplusplus)
#define CAST_LKT_EVENT(i) reinterpret_cast<void *>(static_cast<size_t>(i))
#else
#define CAST_LKT_EVENT(i) ((void *)(size_t)i)
#endif
#define LKT_PLAY_STOP   CAST_LKT_EVENT(__LKT_PLAY_STOP)
#define LKT_PLAY_PLAY   CAST_LKT_EVENT(__LKT_PLAY_PLAY)
#define LKT_PLAY_PAUSE  CAST_LKT_EVENT(__LKT_PLAY_PAUSE)
#define LKT_PLAY_TOGGLE CAST_LKT_EVENT(__LKT_PLAY_TOGGLE)

#define LKT_DB_UPDATING_PROGRESS CAST_LKT_EVENT(__LKT_DB_UPDATING_PROGRESS)
#define LKT_DB_UPDATING_FINISHED CAST_LKT_EVENT(__LKT_DB_UPDATING_FINISHED)

typedef enum {
    LKT_EVENT_NULL            = 0,         // NULL
    LKT_EVENT_PLAY_POS        = (1 << 1),  // size_t
    LKT_EVENT_PLAY_FILE       = (1 << 2),  // XXX: UNUSED
    LKT_EVENT_PLAY_NEXT       = (1 << 3),  // NULL
    LKT_EVENT_PLAY_PREV       = (1 << 4),  // NULL
    LKT_EVENT_PLAY_TOGGLE     = (1 << 5),  // size_t, `LKT_PLAY_.*`
    LKT_EVENT_PROP_VOL        = (1 << 6),  // size_t
    LKT_EVENT_PROP_DUR        = (1 << 7),  // size_t
    LKT_EVENT_PROP_TIME       = (1 << 8),  // size_t
    LKT_EVENT_SKIP_CURRENT    = (1 << 9),  // NULL
    LKT_EVENT_DB_UPDATING     = (1 << 10), // size_t, `LKT_DB_UPDATING_*`
    LKT_EVENT_DB_UPDATE_TOTAL = (1 << 11), // size_t, increments udapte_total
    LKT_EVENT_DB_UPDATE_TICK  = (1 << 12), // NULL, increment the `update_current`
    LKT_EVENT_TOUCH_KARA      = (1 << 13), // size_t, kara id
} LKT_EVENT_TYPE;

#define LKT_EVENT_PLAY                                                                      \
    (LKT_EVENT_PLAY_POS | LKT_EVENT_PLAY_FILE | LKT_EVENT_PLAY_NEXT | LKT_EVENT_PLAY_PREV | \
     LKT_EVENT_PLAY_TOGGLE | LKT_EVENT_SKIP_CURRENT)
#define LKT_EVENT_PROP (LKT_EVENT_PROP_VOL | LKT_EVENT_PROP_DUR | LKT_EVENT_PROP_TIME)
#define LKT_EVENT_UPDATE                                                            \
    (LKT_EVENT_DB_UPDATING | LKT_EVENT_DB_UPDATE_TICK | LKT_EVENT_DB_UPDATE_TOTAL | \
     LKT_EVENT_TOUCH_KARA)

typedef struct {
    unsigned int type;
    void *attr;
} lkt_event;

struct queue {
    pthread_mutex_t lock;

    volatile lkt_event *volatile contents;
    volatile size_t size;
    volatile size_t last;

    volatile unsigned int available;
};

/* Initialize the queue. Can't fail if the passed argument is not NULL.
 * WARN: In case of out of memory this function will abort the process. */
int lkt_queue_new(struct queue *ret);

/* Free the queue. Can't fail. */
void lkt_queue_free(struct queue *queue);

/* Is an event of the following type present in the queue?
 * Returns 0 if the type is not present, 1 otherwise. */
bool lkt_queue_has_event(struct queue *queue, LKT_EVENT_TYPE type);

/* Send an event into the queue */
void lkt_queue_send(struct queue *queue, LKT_EVENT_TYPE _type, void *_attr);

lkt_event lkt_queue_handle(struct queue *queue);

void lkt_queue_make_available(struct queue *queue, LKT_EVENT_TYPE type);

#if defined(__cplusplus)
}
#endif
#endif /* LKT_QUEUE_H__ */
